@@ -1,7 +1,6 @@
-import os,sys
-sys.path.append(os.getcwd())
-import linkcheck
+import os, linkcheck
 config = linkcheck.Config.Configuration()
+config.addLogger('test', linkcheck.test_support.TestLogger)
 config['recursionlevel'] = 1
 config['log'] = config.newLogger('test')
 config["anchors"] = 1
@@ -10,5 +9,5 @@ config.disableThreading()
 htmldir = "test/html"
 for file in ('https.html',):
     url = os.path.join(htmldir, file)
-    config.appendUrl(linkcheck.UrlData.GetUrlDataFrom(url, 0))
+    config.appendUrl(linkcheck.UrlData.GetUrlDataFrom(url, 0, config))
 linkcheck.checkUrls(config)
